
# Import the modules needed to run the script.
from lxml import etree

# Main definition - constants
port_type_display_str = {
    "NU": "NOT_USED       ",
    "DI": "DIGITAL_INPUT  ",
    "DO": "DIGITAL_OUTPUT ",
    "AI": "ANALOG_INPUT   ",
    "AO": "ANALOG_OUTPUT  ",
}

port_type_to_bin_conv = {
    "NU": "00",
    "DI": "00",
    "DO": "01",
    "AI": "10",
    "AO": "11",
}

port_type_h_txt = {
    "NU":  "BINARY_INPUT  ",
    "DI":  "BINARY_INPUT  ",
    "DO":  "BINARY_OUTPUT ",
    "AI":  "ANALOG_INPUT  ",
    "AO":  "PWM_OUTPUT    ",
}

NUM_PORTS =16


# =======================
#     I/O Port class
# =======================

class IOPort:
    def __init__(self, i):
        """Representation of a I/O port
        :param i: Index to the ports in a board
        """
        self.name= "IO_PORT" + ("{:0>2d}".format(i))
        self.options=["NU"]
        self.actual_function="NU"
        self.description="node_"+ ("{:0>2d}".format(i))
            
    def add_available_option(self, option):
        """Add an available option (NU/DI/DO/AI/AO) to the port
        :param option: Available option to add
        """
        self.options.append(option)
        
    def check_selected_option(self, option):
        """Check if an option in is the list of available option for the port
        :param option: Option to check
        """
        return(option in self.options)
            
    def set_actual_function(self, option):
        """Set the selected option for the port
        :param option: Option to set
        """
        self.actual_function=option
                    
    def set_description(self, decription):
        """Set a description for the port
        :param decription: Description to set
        """
        self.description=decription
        
    def get_str_port_options(self):
        """Get a string with the list o available options for the port"""
        num_options=len(self.options)
        txt_options="( "
        for i in range(num_options):
            txt_options=txt_options+str(self.options[i])
            if (i!=num_options-1):
                txt_options=txt_options+"/"
        txt_options=txt_options+" )"
        return txt_options
        

# =======================
#     I/O Board class
# =======================

class IOBoard:
    def __init__(self):
        """Representation of a I/O board"""
    
        #Put default values to board
        self.name="IOboard"
        self.description="IOboard description"
        self.product_code=0
        self.used_ports=0
        self.IO_port_table = []  
        
        for i in range(0, NUM_PORTS):
            self.IO_port_table.append(IOPort(i))
            self.IO_port_table[i].add_available_option("DI")
            self.IO_port_table[i].add_available_option("DO")
    
        self.IO_port_table[2].add_available_option("AO")
        self.IO_port_table[3].add_available_option("AO")
        self.IO_port_table[4].add_available_option("AO")
        self.IO_port_table[5].add_available_option("AO")
    
        self.IO_port_table[8].add_available_option("AI")
        self.IO_port_table[9].add_available_option("AI")
        self.IO_port_table[10].add_available_option("AI")
        self.IO_port_table[11].add_available_option("AI")
       
    
    def set_name(self, name):
        """Set a name for the board
        :param name: Name to set
        """
        self.name=name
    
    def set_description(self, description):
        """Set a description for the board
        :param decription: Description to set
        """
        self.description=description
        
    def _calculate_used_ports_id(self):
        """Calculate the used ports (0xNNNN) for the board to generate the code for the specific configuration"""
        bin_str=""
        for i in range((NUM_PORTS-1), -1, -1):
            if (self.IO_port_table[i].actual_function=="NU"):
                bin_str=bin_str + "0"
            else:
                bin_str=bin_str + "1"
        self.used_ports="{0:0>4X}".format(int(bin_str, 2))
       
    def _calculate_product_id(self):   
        """Calculate the product code (0xNNNNNNNN) for the board to generate the code for the specific configuration"""
        bin_str=""
        for i in range((NUM_PORTS-1), -1, -1):
            bin_str=bin_str + (str(port_type_to_bin_conv[self.IO_port_table[i].actual_function]))
        self.product_code="{0:0>8X}".format(int(bin_str, 2))
           
    def check_port_number(self,port_num):
        """Chech if a port number is within the portss range
        :param port_num: Number to check
        """
        return (port_num>=0 and port_num<NUM_PORTS)
            
    def get_filename_str(self):
        """Generate a string to create the filename of the board including name and product code"""
        filename=self.name + "_" + self.product_code + "_" + self.used_ports 
        return filename
    
    def generate_xml(self):
        """Generate a xml file to run the board in the Lagarto server for the specific configuration"""
        xml_device = etree.Element('device')
        xml_developer = etree.SubElement(xml_device, "developer").text = "apelarda" 
        xml_product = etree.SubElement(xml_device, "product").text = self.name + " - " + self.description
        xml_pwrdownmode = etree.SubElement(xml_device, "pwrdownmode").text = "false" 
        xml_regular = etree.SubElement(xml_device, "regular")
        
        for i in range(0, NUM_PORTS):
            if (self.IO_port_table[i].actual_function!="NU"):
                node_txt = self.IO_port_table[i].description
                if(self.IO_port_table[i].actual_function=="DI"): 
                    type_txt="bin"
                    txt_pos='0.0'
                    txt_size='1'
                else:
                    if (self.IO_port_table[i].actual_function=="DO"):
                       type_txt="bin"
                       txt_pos='0.0'
                       txt_size='0.1'
                    else:
                        type_txt="num"
                        txt_pos='0'
                        txt_size='1'
                
                if(self.IO_port_table[i].actual_function=="DI" or self.IO_port_table[i].actual_function=="AI"):
                    dir_txt="inp"
                else:
                    dir_txt="out"

                #print(("{:0>2d}".format(i))+"---"+self.IO_port_table[i].actual_function+"---"+dir_txt)   
                
                xml_reg = etree.SubElement(xml_regular, "reg",name = self.IO_port_table[i].name, id = str(11+i) )
                xml_endpoint = etree.SubElement(xml_reg, "endpoint",name = node_txt , type=type_txt, dir = dir_txt )
                xml_position = etree.SubElement(xml_endpoint, "position").text = txt_pos
                xml_size = etree.SubElement(xml_endpoint, "size").text = txt_size

        s = etree.tostring(xml_device, xml_declaration=True, pretty_print=True)
        
        filename=self.get_filename_str() + "/" + self.get_filename_str() + ".xml"
        f = open(filename,'w'); 
        ss=s.decode('utf-8')
        f.write(ss)
        f.close()
     


    def generate_define_device_io_header(self):
        """Generate a header file to compile the arduino sketch for the specific configuration"""
        filename = self.get_filename_str() + "/define_device_io.h"
        f = open(filename,'w');
        f.write("#ifndef _DEFINE_IO_DEVICE_H\n")
        f.write("#define _DEFINE_IO_DEVICE_H\n")
        f.write('#include "define_type.h"\n')
        f.write("#define NUM_IO_PORTS 16\n")
        
        for i in range(0, NUM_PORTS):
            f.write("#define " + self.IO_port_table[i].name + "  " + port_type_h_txt[self.IO_port_table[i].actual_function] + "\n")
            
        f.write("#define PROD_ID 0x" + self.product_code+"L   //" + str(long(self.product_code, 16))+"\n")
        f.write("#endif")

        f.close()    
